//---------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "OptionDialog.h"
//--------------------------------------------------------------------- 
#pragma link "CSPIN"
#pragma resource "*.dfm"

#pragma package(smart_init)

TOptionDlg *OptionDlg;
//---------------------------------------------------------------------

// HU{X indent-2.2.9 oC

/*
 * This structure stores all the user options that come from (e.g.)
 * command line flags.
 */
typedef struct user_options_st
{
  int verbose;                  /* when true, non-essential error messages are
                                   printed */
  int use_tabs;                 /* does indent convert tabs to spaces or not */
  int tabsize;                  /* The number of columns a tab character
                                   generates. */
  int use_stdout;               /* Where does output go. */
  int space_sp_semicolon;       /* If true, a space is inserted between if,
                                   while, or for, and a semicolon for
                                   examplewhile (*p++ == ' ') ; */
  int swallow_optional_blanklines;
  int star_comment_cont;        /* true if comment continuation lines should
                                   have stars at the beginning of each line. */
  int struct_brace_indent;      /* Indentation level to be used for a '{'
                                   directly following a struct, union or enum */
  int space_after_while;        /* Put a space after `while' */
  int space_after_if;           /* Put a space after `if' */
  int space_after_for;          /* Put a space after `for' */
  int procnames_start_line;     /* if true, the names of procedures being
                                   defined get placed in column 1 (ie. a
                                   newline is placed between the type of the
                                   procedure and its name) */
  int parentheses_space;        /* If true, parentheses will look like: ( foo
                                   ) rather than (foo) */
  int preserve_mtime;           /* True when the modification time of the files 
                                   should be preserved. */
  int paren_indent;             /* set to the indentation per open parens */
  int proc_calls_space;         /* If true, procedure calls look like: foo
                                   (bar) rather than foo(bar) */
  int leave_preproc_space;      /* if true, leave the spaces between '#' and
                                   preprocessor commands. */
  int force_preproc_width;      /* if this is >0, this becomes the preproc
                                   indent-level */
  int lineup_to_parens;         /* if true, continued code within parens will
                                   be lined up to the open paren */
  int honour_newlines;          /* True when positions at which we read a
                                   newline in the input file, should get a high 
                                   priority to break long lines at. */
  int format_comments;          /* If any comments are to be reformatted */
  int format_col1_comments;     /* If comments which start in column 1 are to
                                   be magically reformatted */
  int extra_expression_indent;  /* True if continuation lines from the
                                   expression part of "if(e)", "while(e)",
                                   "for(e;e;e)" should be indented an extra tab 
                                   stop so that they don't conflict with the
                                   code that follows */
  int ljust_decl;               /* true if declarations should be left
                                   justified */
  int cast_space;               /* If true, casts look like: (char *) bar
                                   rather than (char *)bar */
  int cuddle_else;              /* true if else should cuddle up to '}' */
  int cuddle_do_while;          /* true if '}' should cuddle up to while in do
                                   loop */
  int comment_delimiter_on_blankline;
  int blank_after_sizeof;       /* true iff a blank should always be inserted
                                   after sizeof */
  int break_function_decl_args; /* true if declarations should have args on new 
                                   lines */
  int break_function_decl_args_end;     /* true if declarations should have ")" 
                                           after args on new lines */
  int leave_comma;              /* if true, never break declarations after
                                   commas */
  int break_before_boolean_operator;    /* True when we prefer to break a long
                                           line before a '&&' or '||', instead
                                           of behind it. */
  int blanklines_before_blockcomments;
  int blanklines_after_declarations;
  int blanklines_after_procs;
  int blanklines_after_declarations_at_proctop; /* This is vaguely similar to
                                                   blanklines_after_declarations
                                                   except that it only applies to
                                                   the first set of declarations
                                                   in a procedure (just after the
                                                   first '{') and it causes a
                                                   blank line to be generated even 
                                                   if there are no declarations */
  int blanklines_around_conditional_compilation;
  int comment_max_col;
  int max_col;                  /* the maximum allowable line length */
  int ind_size;                 /* The size of one indentation level in spaces. 
                                 */
  int indent_parameters;        /* Number of spaces to indent parameters.  */
  int decl_indent;              /* column to indent declared identifiers to */
  int unindent_displace;        /* comments not to the right of code will be
                                   placed this many indentation levels to the
                                   left of code */
  int else_endif_col;           /* The column in which comments to the right of 
                                   #else and #endif should start. */
  int case_indent;              /* The distance to indent case labels from the
                                   switch statement */
  int continuation_indent;      /* set to the indentation between the edge of
                                   code and continuation lines in spaces */
  int decl_com_ind;             /* the column in which comments after
                                   declarations should be put */
  int case_brace_indent;        /* Indentation level to be used for a '{'
                                   directly following a case label. */
  int c_plus_plus;              /* True if we're handling C++ code. */
  int com_ind;                  /* the column in which comments to the right of 
                                   code should start */
  int braces_on_struct_decl_line;       /* when true, brace should be on same
                                           line as the struct declaration */
  int braces_on_func_def_line;  /* when true, brace should be on same line as
                                   the function definition */
  int btype_2;                  /* when true, brace should be on same line as
                                   if, while, etc */
  int brace_indent;             /* number of spaces to indent braces from the
                                   suround if, while, etc. in -bl (bype_2 == 0) 
                                   code */
  int expect_output_file;       /* Means "-o" was specified. */
} user_options_ty;

#define DEFAULT_RIGHT_MARGIN 78
#define DEFAULT_RIGHT_COMMENT_MARGIN 78

#define KR_SETTINGS_STRING (int *) \
     "-nbad\0-bap\0-nbc\0-bbo\0-hnl\0-br\0-brs\0-c33\0-cd33\0" \
     "-ncdb\0-ce\0-ci4\0-cli0\0-d0\0-di1\0-nfc1\0-i4\0-ip0\0-l75\0-lp\0" \
     "-npcs\0-nprs\0-npsl\0-sai\0-saf\0-saw\0-cs\0-nsc\0-nsob\0-nfca\0-cp33\0-nss\0"

#define GNU_SETTINGS_STRING  (int *) \
     "-nbad\0-bap\0-bbo\0-hnl\0-nbc\0-bl\0-bls\0-ncdb\0-cs\0-nce\0" \
     "-di2\0-ndj\0-nfc1\0-i2\0-ip5\0-lp\0-pcs\0-nprs\0-psl\0-nsc\0-sai\0-saf\0-saw\0-nsob\0" \
     "-bli2\0-cp1\0-nfca\0"

#define ORIG_SETTINGS_STRING (int *) \
     "-nbap\0-nbad\0-bbo\0-hnl\0-bc\0-br\0-brs\0-c33\0-cd33\0-cdb\0" \
     "-ce\0-ci4\0-cli0\0-cp33\0-di16\0-fc1\0-fca\0-i4\0-l75\0-lp\0-npcs\0-nprs\0" \
     "-psl\0-sc\0-sai\0-saf\0-saw\0-nsob\0-nss\0-ts8\0"


/* profile types */
typedef enum profile
{
  PRO_BOOL,                     /* boolean */
  PRO_INT,                      /* integer */
  PRO_IGN,                      /* ignore it */
//  PRO_KEY,                      /* -T switch */
  PRO_SETTINGS,                 /* bundled set of settings */
//  PRO_PRSTRING,                 /* Print string and exit */
//  PRO_FUNCTION                  /* Call the associated function. */
} profile_ty;


/* profile specials for booleans */
typedef enum on_or_off
{
  ONOFF_NA,                     /* Not applicable.  Used in table for
                                   non-booleans.  */
  OFF,                          /* This option turns on the boolean variable in 
                                   question.  */
  ON                            /* it turns it off */
} on_or_off_ty;

/* Explicit flags for each option.  */
static int exp_T = 0;
static int exp_bacc = 0;
static int exp_badp = 0;
static int exp_bad = 0;
static int exp_bap = 0;
static int exp_bbb = 0;
static int exp_bbo = 0;
static int exp_bc = 0;
static int exp_bl = 0;
static int exp_blf = 0;
static int exp_bli = 0;
static int exp_bls = 0;
static int exp_bs = 0;
static int exp_c = 0;
static int exp_cbi = 0;
static int exp_cdb = 0;
static int exp_cd = 0;
static int exp_cdw = 0;
static int exp_ce = 0;
static int exp_ci = 0;
static int exp_cli = 0;
static int exp_cp = 0;
static int exp_cpp = 0;
static int exp_cs = 0;
static int exp_d = 0;
static int exp_bfda = 0;
static int exp_bfde = 0;
static int exp_di = 0;
static int exp_dj = 0;
static int exp_eei = 0;
static int exp_fc1 = 0;
static int exp_fca = 0;
static int exp_gnu = 0;
static int exp_hnl = 0;
static int exp_i = 0;
static int exp_ip = 0;
static int exp_kr = 0;
static int exp_l = 0;
static int exp_lc = 0;
static int exp_lp = 0;
static int exp_lps = 0;
static int exp_nip = 0;
static int exp_o = 0;
static int exp_orig = 0;
static int exp_pcs = 0;
static int exp_pi = 0;
static int exp_pmt = 0;
static int exp_pro = 0;
static int exp_prs = 0;
static int exp_psl = 0;
static int exp_ppi = 0;         /* force preprocessor indent at width... */
static int exp_sai = 0;
static int exp_saf = 0;
static int exp_saw = 0;
static int exp_sbi = 0;
static int exp_sc = 0;
static int exp_sob = 0;
static int exp_ss = 0;
static int exp_st = 0;
static int exp_ts = 0;
static int exp_ut = 0;
static int exp_v = 0;
static int exp_version = 0;

/* The following structure is controlled by command line parameters and
 * their meaning is explained in indent.h.  */

user_options_ty settings = { 0 };

/* N.B.: because of the way the table here is scanned, options whose names
 * are substrings of other options must occur later; that is, with -lp vs -l,
 * -lp must be first.  Also, while (most) booleans occur more than once, the
 * last default value is the one actually assigned. */

typedef struct pro
{
  char *p_name;                 /* option name, e.g. "bl", "cli" */
  profile_ty p_type;            /* profile type */
  int p_default;                /* the default value (if PRO_BOOL or PRO_INT) */

  on_or_off_ty p_special;       /* If p_type == PRO_BOOL, ON or OFF to tell how 
                                   this switch affects the variable. Not used
                                   for other p_type's.  */

  int *p_obj;                   /* if p_type == PRO_SETTINGS, a (char *)
                                   pointing to a list of the switches to set,
                                   separated by NULLs, terminated by 2 NULLs.
                                   if p_type ==PRO_BOOL or PRO_INT, the address 
                                   of the variable that gets set by the option. 
                                   if p_type == PRO_PRSTRING, a (char )
                                   pointing to the string. if p_type ==
                                   PRO_FUNCTION, a pointer to a function to be 
                                   called. */

  int *p_explicit;              /* Points to a nonzero value (allocated
                                   statically for all options) if the option
                                   has been specified explicitly.  This is
                                   necessary because for boolean options, the
                                   options to set and reset the variable must
                                   share the explicit flag. */


  TObject *obj;                 /* PRO_BOOL: pointing to a TCheckBox; PRO_INT:
                                   pointing to a TCSpinEdit */

} pro_ty;

#define PRE(x) 0                // (OptionDlg->x)


/* Changed to make GNU style the default. */
//const
pro_ty pro[] = {
//{"version", PRO_PRSTRING, 0, ONOFF_NA, (int *)VERSION, &exp_version},
//  {"v", PRO_BOOL, false, ON, &settings.verbose, &exp_v},
  {"ut", PRO_BOOL, true, ON, &settings.use_tabs, &exp_ut, PRE(ut)},
  {"ts", PRO_INT, 8, ONOFF_NA, &settings.tabsize, &exp_ts, PRE(ts)},
//  {"st", PRO_BOOL, false, ON, &settings.use_stdout, &exp_st,PRE(st)},
  {"ss", PRO_BOOL, false, ON, &settings.space_sp_semicolon, &exp_ss, PRE(ss)},
  {"sob", PRO_BOOL, false, ON, &settings.swallow_optional_blanklines,
   &exp_sob, PRE(sob)},
  {"sc", PRO_BOOL, false, ON, &settings.star_comment_cont, &exp_sc, PRE(sc)},
  {"sbi", PRO_INT, 0, ONOFF_NA, &settings.struct_brace_indent, &exp_sbi,
   PRE(sbi)},
  {"saw", PRO_BOOL, true, ON, &settings.space_after_while, &exp_saw,
   PRE(saw)},
  {"sai", PRO_BOOL, true, ON, &settings.space_after_if, &exp_sai, PRE(sai)},
  {"saf", PRO_BOOL, true, ON, &settings.space_after_for, &exp_saf, PRE(saf)},
  {"psl", PRO_BOOL, true, ON, &settings.procnames_start_line, &exp_psl,
   PRE(psl)},
  {"prs", PRO_BOOL, false, ON, &settings.parentheses_space, &exp_prs,
   PRE(prs)},
//#ifdef PRESERVE_MTIME
//  {"pmt", PRO_BOOL, false, ON, &settings.preserve_mtime, &exp_pmt},
//#endif
  {"pi", PRO_INT, -1, ONOFF_NA, &settings.paren_indent, &exp_pi, PRE(pi)},
  {"pcs", PRO_BOOL, true, ON, &settings.proc_calls_space, &exp_pcs, PRE(pcs)},
  {"orig", PRO_SETTINGS, 0, ONOFF_NA, ORIG_SETTINGS_STRING, &exp_orig},
  {"o", PRO_BOOL, false, ON, &settings.expect_output_file, &exp_o},
//  {"nv", PRO_BOOL, false, OFF, &settings.verbose, &exp_v},
  {"nut", PRO_BOOL, true, OFF, &settings.use_tabs, &exp_ut, PRE(ut)},
  {"nss", PRO_BOOL, false, OFF, &settings.space_sp_semicolon, &exp_ss,
   PRE(ss)},
  {"nsob", PRO_BOOL, false, OFF, &settings.swallow_optional_blanklines,
   &exp_sob, PRE(sob)},
  {"nsc", PRO_BOOL, false, OFF, &settings.star_comment_cont, &exp_sc,
   PRE(sc)},
  {"nsaw", PRO_BOOL, true, OFF, &settings.space_after_while, &exp_saw,
   PRE(saw)},
  {"nsai", PRO_BOOL, true, OFF, &settings.space_after_if, &exp_sai, PRE(sai)},
  {"nsaf", PRO_BOOL, true, OFF, &settings.space_after_for, &exp_saf,
   PRE(saf)},
  {"npsl", PRO_BOOL, true, OFF, &settings.procnames_start_line, &exp_psl,
   PRE(psl)},
  {"nprs", PRO_BOOL, false, OFF, &settings.parentheses_space, &exp_prs,
   PRE(prs)},
  {"npro", PRO_IGN, 0, ONOFF_NA, 0, &exp_pro},
//#ifdef PRESERVE_MTIME
//  {"npmt", PRO_BOOL, false, OFF, &settings.preserve_mtime, &exp_pmt},
//#endif
  {"npcs", PRO_BOOL, true, OFF, &settings.proc_calls_space, &exp_pcs,
   PRE(pcs)},
  {"nlps", PRO_BOOL, false, OFF, &settings.leave_preproc_space, &exp_lps,
   PRE(lps)},
  {"nlp", PRO_BOOL, true, OFF, &settings.lineup_to_parens, &exp_lp, PRE(lp)},
  {"nip", PRO_SETTINGS, 0, ONOFF_NA, (int *)"-ip0\0", &exp_nip},
  {"nhnl", PRO_BOOL, true, OFF, &settings.honour_newlines, &exp_hnl,
   PRE(hnl)},
  {"nfca", PRO_BOOL, false, OFF, &settings.format_comments, &exp_fca,
   PRE(fca)},
  {"nfc1", PRO_BOOL, false, OFF, &settings.format_col1_comments, &exp_fc1,
   PRE(fcl)},
  {"neei", PRO_BOOL, false, OFF, &settings.extra_expression_indent, &exp_eei},
  {"ndj", PRO_BOOL, false, OFF, &settings.ljust_decl, &exp_dj},
  {"ncs", PRO_BOOL, true, OFF, &settings.cast_space, &exp_cs, PRE(cs)},
  {"nce", PRO_BOOL, false, OFF, &settings.cuddle_else, &exp_ce, PRE(ce)},
  {"ncdw", PRO_BOOL, false, OFF, &settings.cuddle_do_while, &exp_cdw,
   PRE(cdw)},
  {"ncdb", PRO_BOOL, false, OFF, &settings.comment_delimiter_on_blankline,
   &exp_cdb, PRE(cdb)},
  {"nbs", PRO_BOOL, false, OFF, &settings.blank_after_sizeof, &exp_bs,
   PRE(bs)},
  {"nbfda", PRO_BOOL, false, OFF, &settings.break_function_decl_args,
   &exp_bfda, PRE(bfda)},
  {"nbfde", PRO_BOOL, false, OFF, &settings.break_function_decl_args_end,
   &exp_bfde, PRE(bfde)},
  {"nbc", PRO_BOOL, true, ON, &settings.leave_comma, &exp_bc, PRE(bc)},
  {"nbbo", PRO_BOOL, true, OFF, &settings.break_before_boolean_operator,
   &exp_bbo, PRE(bbo)},
  {"nbbb", PRO_BOOL, false, OFF, &settings.blanklines_before_blockcomments,
   &exp_bbb, PRE(bbb)},
  {"nbap", PRO_BOOL, true, OFF, &settings.blanklines_after_procs, &exp_bap,
   PRE(bap)},
  {"nbadp", PRO_BOOL, false, OFF,
   &settings.blanklines_after_declarations_at_proctop, &exp_badp},
  {"nbad", PRO_BOOL, false, OFF, &settings.blanklines_after_declarations,
   &exp_bad, PRE(bad)},
  {"nbacc", PRO_BOOL, false, OFF,
   &settings.blanklines_around_conditional_compilation, &exp_bacc},
  {"lps", PRO_BOOL, false, ON, &settings.leave_preproc_space, &exp_lps,
   PRE(lps)},
  {"lp", PRO_BOOL, true, ON, &settings.lineup_to_parens, &exp_lp, PRE(lp)},
  {"lc", PRO_INT, DEFAULT_RIGHT_COMMENT_MARGIN, ONOFF_NA,
   &settings.comment_max_col, &exp_lc, PRE(lc)},
  {"l", PRO_INT, DEFAULT_RIGHT_MARGIN, ONOFF_NA, &settings.max_col, &exp_l,
   PRE(l)},
  {"kr", PRO_SETTINGS, 0, ONOFF_NA, KR_SETTINGS_STRING, &exp_kr},
  {"ip", PRO_INT, 5, ONOFF_NA, &settings.indent_parameters, &exp_ip, PRE(ip)},
  {"i", PRO_INT, 2, ONOFF_NA, &settings.ind_size, &exp_i, PRE(i)},
  {"hnl", PRO_BOOL, true, ON, &settings.honour_newlines, &exp_hnl, PRE(hnl)},
//  {"h", PRO_FUNCTION, 0, ONOFF_NA, (int *)usage, &exp_version},
  /*
   * This is now the default. 
   */
  {"gnu", PRO_SETTINGS, 0, ONOFF_NA, GNU_SETTINGS_STRING, &exp_gnu},
  {"fca", PRO_BOOL, false, ON, &settings.format_comments, &exp_fca, PRE(fca)},
  {"fc1", PRO_BOOL, false, ON, &settings.format_col1_comments, &exp_fc1,
   PRE(fcl)},
  {"eei", PRO_BOOL, false, ON, &settings.extra_expression_indent, &exp_eei},
  {"dj", PRO_BOOL, false, ON, &settings.ljust_decl, &exp_dj},
  {"di", PRO_INT, 2, ONOFF_NA, &settings.decl_indent, &exp_di, PRE(di)},
  {"d", PRO_INT, 0, ONOFF_NA, &settings.unindent_displace, &exp_d, PRE(d)},
  {"cs", PRO_BOOL, true, ON, &settings.cast_space, &exp_cs, PRE(cs)},
  {"cp", PRO_INT, 1, ONOFF_NA, &settings.else_endif_col, &exp_cp, PRE(cp)},
  {"cli", PRO_INT, 0, ONOFF_NA, &settings.case_indent, &exp_cli, PRE(cli)},
  {"ci", PRO_INT, 0, ONOFF_NA, &settings.continuation_indent, &exp_ci,
   PRE(ci)},
  {"ce", PRO_BOOL, false, ON, &settings.cuddle_else, &exp_ce, PRE(ce)},
  {"cdw", PRO_BOOL, false, ON, &settings.cuddle_do_while, &exp_cdw, PRE(cdw)},
  {"cdb", PRO_BOOL, false, ON, &settings.comment_delimiter_on_blankline,
   &exp_cdb, PRE(cdb)},
  {"cd", PRO_INT, 33, ONOFF_NA, &settings.decl_com_ind, &exp_cd, PRE(cd)},
  {"cbi", PRO_INT, -1, ONOFF_NA, &settings.case_brace_indent, &exp_cbi,
   PRE(cbi)},
  {"c++", PRO_BOOL, false, ON, &settings.c_plus_plus, &exp_cpp},
  {"c", PRO_INT, 33, ONOFF_NA, &settings.com_ind, &exp_c, PRE(c)},
  {"bs", PRO_BOOL, false, ON, &settings.blank_after_sizeof, &exp_bs, PRE(bs)},
  {"brs", PRO_BOOL, false, ON, &settings.braces_on_struct_decl_line,
   &exp_bls, PRE(bls)},
  {"bls", PRO_BOOL, false, OFF, &settings.braces_on_struct_decl_line,
   &exp_bls, PRE(bls)},
  {"brf", PRO_BOOL, false, ON, &settings.braces_on_func_def_line, &exp_blf},
  {"blf", PRO_BOOL, false, OFF, &settings.braces_on_func_def_line, &exp_blf},
  {"bli", PRO_INT, 2, ONOFF_NA, &settings.brace_indent, &exp_bli, PRE(bli)},
  {"br", PRO_BOOL, false, ON, &settings.btype_2, &exp_bl, PRE(bl)},
  {"bl", PRO_BOOL, false, OFF, &settings.btype_2, &exp_bl, PRE(bl)},
  {"bfda", PRO_BOOL, false, ON, &settings.break_function_decl_args,
   &exp_bfda, PRE(bfda)},
  {"bfde", PRO_BOOL, false, ON, &settings.break_function_decl_args_end,
   &exp_bfde, PRE(bfde)},
  {"bc", PRO_BOOL, true, OFF, &settings.leave_comma, &exp_bc, PRE(bc)},
  {"bbo", PRO_BOOL, true, ON, &settings.break_before_boolean_operator,
   &exp_bbo, PRE(bbo)},
  {"bbb", PRO_BOOL, false, ON, &settings.blanklines_before_blockcomments,
   &exp_bbb, PRE(bbb)},
  {"bap", PRO_BOOL, true, ON, &settings.blanklines_after_procs, &exp_bap,
   PRE(bap)},
  {"badp", PRO_BOOL, false, ON,
   &settings.blanklines_after_declarations_at_proctop, &exp_badp},
  {"bad", PRO_BOOL, false, ON, &settings.blanklines_after_declarations,
   &exp_bad, PRE(bad)},
  {"bacc", PRO_BOOL, false, ON,
   &settings.blanklines_around_conditional_compilation, &exp_bacc},
//  {"T", PRO_KEY, 0, ONOFF_NA, 0, &exp_T},
  {"ppi", PRO_INT, 0, ONOFF_NA, &settings.force_preproc_width, &exp_ppi},
  /*
   * Signify end of structure.  
   */
  {0, PRO_IGN, 0, ONOFF_NA, 0, 0, 0}
};

typedef struct long_option_conversion
{
  char *long_name;
  char *short_name;
} long_option_conversion_ty;

const long_option_conversion_ty option_conversions[] = {
  {"version", "version"},
  {"verbose", "v"},
  {"usage", "h"},
  {"use-tabs", "ut"},
  {"tab-size", "ts"},
  {"swallow-optional-blank-lines", "sob"},
  {"struct-brace-indentation", "sbi"},
  {"start-left-side-of-comments", "sc"},
  {"standard-output", "st"},
  {"space-special-semicolon", "ss"},
  {"space-after-while", "saw"},
  {"space-after-procedure-calls", "pcs"},
  {"space-after-parentheses", "prs"},
  {"space-after-if", "sai"},
  {"space-after-for", "saf"},
  {"space-after-cast", "cs"},
  {"remove-preprocessor-space", "nlps"},
  {"procnames-start-lines", "psl"},
#ifdef PRESERVE_MTIME
  {"preserve-mtime", "pmt"},
#endif
  {"paren-indentation", "pi"},
  {"parameter-indentation", "ip"},
  {"output-file", "o"},
  {"output", "o"},
  {"original-style", "orig"},
  {"original", "orig"},
  {"no-verbosity", "nv"},
  {"no-tabs", "nut"},
  {"no-space-after-while", "nsaw"},
  {"no-space-after-parentheses", "nprs"},
  {"no-space-after-if", "nsai"},
  {"no-space-after-function-call-names", "npcs"},
  {"no-space-after-for", "nsaf"},
  {"no-space-after-casts", "ncs"},
  {"no-parameter-indentation", "nip"},
  {"no-extra-expression-indentation", "neei"},
  {"no-comment-delimiters-on-blank-lines", "ncdb"},
  {"no-blank-lines-before-block-comments", "nbbb"},
  {"no-blank-lines-after-procedures", "nbap"},
  {"no-blank-lines-after-procedure-declarations", "nbadp"},
  {"no-blank-lines-after-ifdefs", "nbacc"},
  {"no-blank-lines-after-declarations", "nbad"},
  {"no-blank-lines-after-commas", "nbc"},
  {"no-blank-before-sizeof", "nbs"},
  {"no-Bill-Shannon", "nbs"},
  {"line-length", "l"},
  {"line-comments-indentation", "d"},
  {"left-justify-declarations", "dj"},
  {"leave-preprocessor-space", "lps"},
  {"leave-optional-blank-lines", "nsob"},
  {"kernighan-and-ritchie-style", "kr"},
  {"kernighan-and-ritchie", "kr"},
  {"k-and-r-style", "kr"},
  {"indentation-level", "i"},
  {"indent-level", "i"},
  {"ignore-profile", "npro"},
  {"ignore-newlines", "nhnl"},
  {"honour-newlines", "hnl"},
  {"help", "h"},
  {"gnu-style", "gnu"},
  {"format-first-column-comments", "fc1"},
  {"format-all-comments", "fca"},
  {"extra-expression-indentation", "eei"},
  {"else-endif-column", "cp"},
  {"dont-star-comments", "nsc"},
  {"dont-space-special-semicolon", "nss"},
  {"dont-line-up-parentheses", "nlp"},
  {"dont-left-justify-declarations", "ndj"},
  {"dont-indent-parameters", "nip"},
  {"dont-format-first-column-comments", "nfc1"},
  {"dont-format-comments", "nfca"},
  {"dont-cuddle-else", "nce"},
  {"dont-cuddle-do-while", "ncdw"},
  {"dont-break-procedure-type", "npsl"},
  {"dont-break-function-decl-args", "nbfda"},
  {"dont-break-function-decl-args-end", "nbfde"},
  {"declaration-indentation", "di"},
  {"declaration-comment-column", "cd"},
  {"cuddle-else", "ce"},
  {"cuddle-do-while", "cdw"},
  {"continue-at-parentheses", "lp"},
  {"continuation-indentation", "ci"},
  {"comment-line-length", "lc"},
  {"comment-indentation", "c"},
  {"comment-delimiters-on-blank-lines", "cdb"},
  {"case-indentation", "cli"},
  {"case-brace-indentation", "cbi"},
  {"c-plus-plus", "c++"},
  {"break-function-decl-args", "bfda"},
  {"break-function-decl-args-end", "bfde"},
  {"break-before-boolean-operator", "bbo"},
  {"break-after-boolean-operator", "nbbo"},
  {"braces-on-struct-decl-line", "brs"},
  {"braces-on-func-def-line", "brf"},
  {"braces-on-if-line", "br"},
  {"braces-after-struct-decl-line", "bls"},
  {"braces-after-func-def-line", "blf"},
  {"braces-after-if-line", "bl"},
  {"brace-indent", "bli"},
  {"blank-lines-before-block-comments", "bbb"},
  {"blank-lines-after-procedures", "bap"},
  {"blank-lines-after-procedure-declarations", "badp"},
  {"blank-lines-after-ifdefs", "bacc"},
  {"blank-lines-after-declarations", "bad"},
  {"blank-lines-after-commas", "bc"},
  {"blank-before-sizeof", "bs"},
  {"berkeley-style", "orig"},
  {"berkeley", "orig"},
  {"Bill-Shannon", "bs"},
  {"preprocessor-indentation", "ppi"},
  {0, 0},
};

char *get_longname(char *sname)
{
  const long_option_conversion_ty *opt = option_conversions;

  for(; opt->short_name; opt++)
  {
    if(lstrcmp(sname, opt->short_name) == 0)
      return opt->long_name;
  }
  return 0;
}

/* S1 should be a string.  S2 should be a string, perhaps followed by an
   argument.  Compare the two, returning true if they are equal, and if they
   are equal set *START_PARAM to point to the argument in S2.  */

static BOOLEAN eqin(const char *s1, const char *s2, const char **start_param)
{
  BOOLEAN ret = true;

  while(*s1)
  {
    if(*s1++ != *s2++)
    {
      ret = false;
    }
  }

  *start_param = s2;
  return ret;
}

/* Set the defaults. */

void set_defaults(void)
{
  const pro_ty *p;

  for(p = pro; p->p_name; p++)
  {
    if((p->p_type == PRO_BOOL && p->p_special == ON) || p->p_type == PRO_INT)
    {
      *p->p_obj = p->p_default;
    }
  }
}

/* Strings which can prefix an option, longest first. */

static char *option_prefixes[] = {
  "--",
  "-",
  "+",
  0
};

static int option_prefix(const char *arg)
{
  char **prefixes = option_prefixes;
  char *this_prefix;            // = *prefixes;
  const char *argp;             // = arg;
  int ret = 0;

  do
  {
    this_prefix = *prefixes;
    argp = arg;

    while(*this_prefix == *argp)
    {
      this_prefix++;
      argp++;
    }

    if(*this_prefix == '\0')
    {
      ret = this_prefix - *prefixes;
      break;
    }

  }
  while(*++prefixes);

  return ret;
}

/* Process an option ARG (e.g. "-l60").  PARAM is a possible value
 * for ARG, if PARAM is nonzero.  EXPLICT should be nonzero iff the
 * argument is being explicitly specified (as opposed to being taken from a
 * PRO_SETTINGS group of settings).
 *
 * Returns 1 if the option had a value, returns 0 otherwise.
 *
 * 2002-06-13 D.Ingamells. Fixed check for int param without an int argument.
 */

int set_option(const char *option, const char *param, int explicit1)
{
  const pro_ty *p = pro;
  const char *param_start = NULL;
  int option_length = option_prefix(option);
  int val = 0;
  BOOLEAN found = false;

  if(option_length > 0)
  {
    if((option_length == 1) && (*option == '-'))
    {
      /*
       * Short option prefix 
       */
      option++;

      for(p = pro; p->p_name; p++)
      {
        if((*p->p_name == *option) && eqin(p->p_name, option, &param_start))
        {
          found = true;
          break;
        }
      }
    }
    else
    {
      /*
       * Long prefix 
       */
      const long_option_conversion_ty *o = option_conversions;

      option += option_length;

      while(o->short_name)
      {
        if(eqin(o->long_name, option, &param_start))
        {
          break;
        }

        o++;
      }

      /*
       * Searching twice means we don't have to keep the two tables in sync. 
       */

      if(o->short_name)
      {
        for(p = pro; p->p_name; p++)
        {
          if(!strcmp(p->p_name, o->short_name))
          {
            found = true;
            break;
          }
        }
      }
    }
  }

  if(!found)
  {
    // fprintf (stderr, _("indent: unknown option \"%s\"\n"), option - 1);
    // exit (invocation_error);
  }
  else
  {
    /*
     * If the parameter has been explicitly specified, we don't want a group
     * of bundled settings to override the explicit setting.  
     */

    if(settings.verbose)
    {
      // fprintf (stderr, _("option: %s\n"), p->p_name);
    }

    if(explicit1 || !*(p->p_explicit))
    {
      if(explicit1)
      {
        *(p->p_explicit) = 1;
      }

      switch (p->p_type)
      {

        // case PRO_PRSTRING:
        /*
         * This is not really an error, but zero exit values are returned
         * only when code has been successfully formatted. 
         */
        // printf (_("GNU indent %s\n"), (char *) p->p_obj);
        // exit (invocation_error);

        // case PRO_FUNCTION:
        // ((void (*)())p->p_obj) ();
        // break;

      case PRO_SETTINGS:
        {
          char *t;              /* current position */

          t = (char *)p->p_obj;

          do
          {
            set_option(t, 0, 0);
            /*
             * advance to character following next NUL 
             */

            while(*t++)
            {
            }
          }
          while(*t);
        }

      case PRO_IGN:
        break;

        /*** /
      case PRO_KEY:
        {
          char *str;

          if (*param_start == 0)
          {
            if (!(param_start = param))
            {
              //arg_missing(option);
            }
            else
            {
              val = 1;
            }
          }

          str = (char *)xmalloc(strlen(param_start) + 1);
          strcpy(str, param_start);
          addkey(str, rw_decl);
        }
        break;
        /***/

      case PRO_BOOL:
        if(p->p_special == OFF)
        {
          *p->p_obj = false;
        }
        else
        {
          *p->p_obj = true;
        }

        break;

      case PRO_INT:
        if(*param_start == '\0')
        {
          param_start = param;

          if(param_start == NULL)
          {
            // arg_missing(option);
            param_start = 0;
          }
          else
          {
            val = 1;
          }
        }

        if(isdigit(*param_start)
           || ((*param_start == '-') && isdigit(*(param_start + 1))))
        {
          *p->p_obj = atoi(param_start);
        }
        else
        {
          /*** /
          fprintf(stderr,
                  _("indent: option ``%s'' requires a numeric parameter\n"),
                  option - 1);
          exit(invocation_error);
          /***/
        }

        break;

        /*** /
      default:
        fprintf(stderr,
                _("indent: set_option: internal error: p_type %d\n"),
                (int)p->p_type);
        exit(invocation_error);
        /***/
      }
    }
  }

  return val;
}



// N .indent.pro Ūo pointer 
char *fileptr;
int filepos;

#define BUFSIZ 1024
#define FILE void
#define EOF 0
#define EOL 0xD
#define EOS 0


inline int getc(FILE * f)
{
  int c = fileptr[filepos];
  filepos++;
  return c;
}



/* The first 2 characters of a c++ comment have been read skip the remainder of the line
 * and return the first character from the next line.
 */

static int skip_cpp_comment(FILE * f)
{

  int i;

  do
  {
    i = getc(f);
  }
  while((i != EOF) && (i != EOL));

  if(i == EOL)
  {
    i = getc(f);
  }

  return i;
}

/*
 * the first 2 characters of a c comment have been read. Read past the
 * remainder of the comment and return the first character after the
 * comment.
 */

static int skip_c_comment(FILE * f)
{
  int i = getc(f);

  do
  {
    /*
     * skip to the next '*' character 
     */

    while((i != EOF) && (i != '*'))
    {
      i = getc(f);
    }


    if(i == EOF)
    {
      // WARNING (_("Profile contains an unterminated comment"), 0, 0);
      break;
    }
    /*
     * current char is '*' so skip over it. 
     */

    i = getc(f);

    /*
     * If the current character is a '/' the comment is over. 
     */
  }
  while(i != '/');

  if(i != EOF)
  {
    i = getc(f);
  }

  return i;
}

/* The starting / of a comment has been read.
 * skip over the rest of the comment and return the
 * first character after the comment.
 */

static int skip_comment(FILE * f)
{
  int i = getc(f);

  switch (i)
  {
  case '/':
    i = skip_cpp_comment(f);
    break;
  case '*':
    i = skip_c_comment(f);
    break;
    // default:
    // WARNING (_("Profile contains unpalatable characters"), 0, 0);
  }

  return i;
}



/*
 * Skip a sequence of space and control characters and return the
 * first character after the sequence.
 */

static int skip_spaces(FILE * f, int first)
{
  int i = first;

  while((i <= ' ') && (i != EOF))
  {
    i = getc(f);
  }

  return i;
}


/* Read a string from the input until the next control character, space or /.
 * Return the first character after the read string.
 */

static int read_string(FILE * f, char *buff, int first)
{
  int i = first;
  char *p = buff;

  while((i != EOF) && (i > ' ') && (i != '/') && (p < buff + BUFSIZ))
  {
    *(p++) = i;
    i = getc(f);
  }

  *p = EOS;

  return i;
}


/* Scan the options in the file f. */

static void scan_profile(FILE * f)
{
  char b0[BUFSIZ];
  char b1[BUFSIZ];
  char *current = b0;

  int i = skip_spaces(f, ' ');

  while(i != EOF)
  {
    if(i == '/')
    {
      /*
       * The start of a comment. 
       */
      i = skip_comment(f);
    }
    else
    {
      i = read_string(f, current, i);

      /*
       * We've scanned something... 
       */

      if(current == b0)
      {
        /*
         * Second buffer still has to be filled. 
         */
        current = b1;
      }
      else if(set_option(b0, b1, 1) == 1)
      {
        /*
         * The option had a parameter, thus both values have been consumed.
         * Reset the 2 buffers to 'empty' 
         */
        current = b0;
      }
      else
      {
        /*
         * Set option consumed one value. Move the other value to the first
         * buffer and go get a new second value. 
         */

        strcpy(b0, b1);
        current = b1;
      }

    }

    i = skip_spaces(f, i);
  }

  if(current != b0)
  {
    set_option(b0, NULL, 1);
  }
}


void reset_settings()
{
  exp_T = 0;
  exp_bacc = 0;
  exp_badp = 0;
  exp_bad = 0;
  exp_bap = 0;
  exp_bbb = 0;
  exp_bbo = 0;
  exp_bc = 0;
  exp_bl = 0;
  exp_blf = 0;
  exp_bli = 0;
  exp_bls = 0;
  exp_bs = 0;
  exp_c = 0;
  exp_cbi = 0;
  exp_cdb = 0;
  exp_cd = 0;
  exp_cdw = 0;
  exp_ce = 0;
  exp_ci = 0;
  exp_cli = 0;
  exp_cp = 0;
  exp_cpp = 0;
  exp_cs = 0;
  exp_d = 0;
  exp_bfda = 0;
  exp_bfde = 0;
  exp_di = 0;
  exp_dj = 0;
  exp_eei = 0;
  exp_fc1 = 0;
  exp_fca = 0;
  exp_gnu = 0;
  exp_hnl = 0;
  exp_i = 0;
  exp_ip = 0;
  exp_kr = 0;
  exp_l = 0;
  exp_lc = 0;
  exp_lp = 0;
  exp_lps = 0;
  exp_nip = 0;
  exp_o = 0;
  exp_orig = 0;
  exp_pcs = 0;
  exp_pi = 0;
  exp_pmt = 0;
  exp_pro = 0;
  exp_prs = 0;
  exp_psl = 0;
  exp_ppi = 0;                  /* force preprocessor indent at width... */
  exp_sai = 0;
  exp_saf = 0;
  exp_saw = 0;
  exp_sbi = 0;
  exp_sc = 0;
  exp_sob = 0;
  exp_ss = 0;
  exp_st = 0;
  exp_ts = 0;
  exp_ut = 0;
  exp_v = 0;
  exp_version = 0;

  set_defaults();
}



//---------------------------------------------------------------------

TStringList *oldprofile;
TStringList *demostring;


void save_profile()
{
  // write options to .indent.pro

  TStringList *file = new TStringList();
  pro_ty *p;
  for(p = pro; p->p_name; p++)
  {
    if(p->obj != 0)
      if(p->p_type == PRO_BOOL && *p->p_obj != p->p_default)
      {
        pro_ty *p2;
        for(p2 = pro; p2->p_name; p2++)
        {
          int on_off = (p2->p_special != OFF);
          if(p2->obj == p->obj && *p->p_obj == on_off)
          {
            char *lname = get_longname(p2->p_name);
            String opt = String("--") + lname;

            if(file->IndexOf(opt) < 0)
              file->Add(opt);

            break;
          }
        }
      }
      else if(p->p_type == PRO_INT && *p->p_obj != p->p_default)
      {
        char *lname = get_longname(p->p_name);
        file->Add(String("--") + String(lname) + String(*p->p_obj));
      }
  }

  String name = OptionDlg->DllPath + ".indent.pro";
  DeleteFile(name);
  file->SaveToFile(name);

  delete file;
}

void indent_tmpfile()
{
  // MessageBox(0,OptionDlg->DllPath.c_str(),"",MB_OK);

  String indent = OptionDlg->DllPath + "indent.exe delete_me.cpp";

  STARTUPINFO StartupInfo;
  ZeroMemory(&StartupInfo, sizeof(STARTUPINFO));
  StartupInfo.cb = sizeof(STARTUPINFO);
  // we are going to set the HIDE-WINDOW flag
  StartupInfo.dwFlags |= STARTF_USESHOWWINDOW;
  StartupInfo.wShowWindow = SW_HIDE;

  char *currdir = NULL;
  if(OptionDlg->DllPath != "")
    currdir = OptionDlg->DllPath.c_str();

  PROCESS_INFORMATION ProcessInfo;
  if(CreateProcess
     (NULL, indent.c_str(), NULL, NULL, TRUE, NORMAL_PRIORITY_CLASS, NULL,
      currdir, &StartupInfo, &ProcessInfo))
  {
    DWORD dwExitStatus = STILL_ACTIVE;
    // loop until indent closes
    do
    {
      if(!GetExitCodeProcess(ProcessInfo.hProcess, &dwExitStatus))
        break;

      Application->ProcessMessages();
    }
    while(dwExitStatus == STILL_ACTIVE);

    CloseHandle(ProcessInfo.hProcess);
    CloseHandle(ProcessInfo.hThread);
  }
  else
    MessageDlg("Couldn't run Indent.exe", mtInformation,
               TMsgDlgButtons() << mbOK, 0);

}


void load_settings_to_dialog()
{
  pro_ty *p;
  for(p = pro; p->p_name; p++)
  {
    String name;
    if(p->p_type == PRO_BOOL && p->p_name[0] == 'n')
      name = p->p_name + 1;
    else
      name = p->p_name;

    p->obj = (TObject *) OptionDlg->FindComponent(name);

    if(p->obj != 0)
      if(p->p_type == PRO_BOOL)
      {
        ((TCheckBox *) p->obj)->Checked = *p->p_obj;
      }
      else if(p->p_type == PRO_INT)
      {
        ((TCSpinEdit *) p->obj)->Value = *p->p_obj;
      }
  }
}

void enable_event()
{
  pro_ty *p;
  for(p = pro; p->p_name; p++)
  {
    if(p->obj != 0)
      if(p->p_type == PRO_BOOL)
      {
        ((TCheckBox *) p->obj)->OnClick = OptionDlg->CheckBoxClick;
      }
      else if(p->p_type == PRO_INT)
      {
        ((TCSpinEdit *) p->obj)->OnChange = OptionDlg->SpinEditChange;
      }
  }
}

void disable_event()
{
  pro_ty *p;
  for(p = pro; p->p_name; p++)
  {
    if(p->obj != 0)
      if(p->p_type == PRO_BOOL)
      {
        ((TCheckBox *) p->obj)->OnClick = NULL;
      }
      else if(p->p_type == PRO_INT)
      {
        ((TCSpinEdit *) p->obj)->OnChange = NULL;
      }
  }
}


//---------------------------------------------------------------------
__fastcall TOptionDlg::TOptionDlg(TComponent * AOwner):TForm(AOwner)
{
  oldprofile = new TStringList();
  demostring = new TStringList();

  // DllPath=".\\";
}

__fastcall TOptionDlg::~TOptionDlg()
{
  delete oldprofile;
  delete demostring;
}

//---------------------------------------------------------------------------

void __fastcall TOptionDlg::FormShow(TObject * Sender)
{
  String ProFile = OptionDlg->DllPath + ".indent.pro";

  if(!FileExists(ProFile))
  {
    oldprofile->Clear();
    oldprofile->SaveToFile(ProFile);    // sy@ӪŪprofile
  }

  oldprofile->LoadFromFile(ProFile);
  reset_settings();

  // load options from .indent.pro
  String opts = oldprofile->Text;
  fileptr = opts.c_str();
  filepos = 0;
  scan_profile(0);

  load_settings_to_dialog();

  enable_event();
}


void __fastcall TOptionDlg::FormHide(TObject * Sender)
{
  disable_event();
}

//---------------------------------------------------------------------------


String demo1 =
  "char *foo;\n" "char *bar;\n"
  "/* Force blank lines after the declarations??? */\n" "\n" "int baz;\n"
  "int\n" "foo ()\n" "{\n" "  puts (\" Hi \");\n" "}\n"
  "/* Force blank lines after procedure bodies??? */\n" "\n"
  "/* Force blank lines before block comments??? */\n"
  "/* Force blank lines before block comments??? */\n" "\n\n\n\n"
  "/* Swallow optional blank lines??? */\n";

String demo2 =
  "int foo()/*Put comments to the right of the declarations in column N.*/\n"
  "{\n"
  "int a=0;/*Put comments to the right of the declarations in column N.*/\n"
  "a++;/*Put comments to the right of code in column N.*/\n}\n"
  "/* Put comment delimiters on blank lines. */\n" "/**********************\n"
  " * Comment in a box!! *\n" " **********************/\n\n" "/*\n"
  "  abcdef rtyui\n" "*/\n\n" "#ifdef PIG/* XX */\n\n" "#else/* XX */\n\n"
  "#endif/* XX */\n\n" "";

String demo3 =
  "typedef struct A{float f1;} _A;\n\n" "int foo(char c){\n" "  int bzz=0;\n\n"
  "  if(bzz==3) bzz++;\n" "  else{ bzz=(int)c;}\n\n"
//"  while(c) c--;\n\n"
  "  for(bzz=3;bzz<10;bzz++) {bzz+=5;}\n\n"
  "  do{bzz-=sizeof(int);c++;}while(c<10);\n\n" "  return bzz;  \n" "}\n" "";

String demo4 =
  "typedef struct A{float f1;} _A;\n\n" "int foo(float x,float y,float z){\n"
  "  int a,b,c,d;\n\n" "  int bzz=0;\n\n" "  return bzz;  \n" "}\n" "";

String demo5 =
  "#if X\n" "#if Y\n" "#define Z 1\n" "#else\n" "#define Z 0\n" "#endif\n"
  "#endif\n\n" "int foo(x,y)float x;float b;{\n" "\tint a,b,c,d;\n\n"
  "\tint\tbzz=0;\n\n" "p1 = first_procedure (second_procedure (p2, p3),\n"
  "     third_procedure (p4, p5),ooo_procedure (p4, p5));\n\n"
  "  return bzz;  \n" "}\n" "";

String demo6 =
  "int x()\n{\n\n"
  "  if (maskisblack && ((maskisblack[0] == '\\0')|| (maskisblack[1] == '\\0'&& ((maskisblack[0] == '0') || (maskisblack[0] == '*')))))\n"
  " a=3;\n\n"
  "  if (maskisblack &&\n((maskisblack[0] == '\\0')||\n(maskisblack[1] == '\\0' &&\n ((maskisblack[0] == '0') || \n(maskisblack[0] == '*')))))\n"
  " b=3;\n" "\nreturn oo;}\n" "";


void __fastcall TOptionDlg::TabSheet1Show(TObject * Sender)
{
  if(Sender == TabSheet1)
    demostring->Text = demo1;
  else if(Sender == TabSheet2)
    demostring->Text = demo2;
  else if(Sender == TabSheet3)
    demostring->Text = demo3;
  else if(Sender == TabSheet4)
    demostring->Text = demo4;
  else if(Sender == TabSheet5)
    demostring->Text = demo5;
  else if(Sender == TabSheet6)
    demostring->Text = demo6;

  String file = DllPath + "delete_me.cpp";

  demostring->SaveToFile(file);
  indent_tmpfile();
  Memo1->Lines->LoadFromFile(file);

  DeleteFile(file);
  DeleteFile(file + "~");
}

//---------------------------------------------------------------------------

void __fastcall TOptionDlg::CheckBoxClick(TObject * Sender)
{
  static bool doing = false;
  if(doing)
    return;
  doing = true;

  // NܪѼƦsJ
  pro_ty *p;
  for(p = pro; p->p_name; p++)
  {
    if(p->obj == Sender)
    {
      *p->p_obj = ((TCheckBox *) Sender)->Checked;
      break;
    }
  }

  save_profile();

  String name = DllPath + "delete_me.cpp";

  demostring->SaveToFile(name);
  indent_tmpfile();
  Memo1->Lines->LoadFromFile(name);

  DeleteFile(name);
  DeleteFile(name + "~");

  doing = false;
}

void __fastcall TOptionDlg::SpinEditChange(TObject * Sender)
{
  static bool doing = false;
  if(doing)
    return;
  doing = true;

  // NܪѼƦsJ
  pro_ty *p;
  for(p = pro; p->p_name; p++)
  {
    if(p->obj == Sender)
    {
      *p->p_obj = ((TCSpinEdit *) Sender)->Value;
      break;
    }
  }

  save_profile();

  String name = DllPath + "delete_me.cpp";

  demostring->SaveToFile(name);
  indent_tmpfile();
  Memo1->Lines->LoadFromFile(name);

  DeleteFile(name);
  DeleteFile(name + "~");

  doing = false;
}

//---------------------------------------------------------------------------

void __fastcall TOptionDlg::BitBtn1Click(TObject * Sender)
{
  save_profile();
}

//---------------------------------------------------------------------------

void __fastcall TOptionDlg::BitBtn2Click(TObject * Sender)
{
  reset_settings();

  String opts = oldprofile->Text;
  fileptr = opts.c_str();
  filepos = 0;
  scan_profile(0);

  save_profile();
}

//---------------------------------------------------------------------------

void __fastcall TOptionDlg::BitBtn3Click(TObject * Sender)
{
  reset_settings();

  String opts = oldprofile->Text;
  fileptr = opts.c_str();
  filepos = 0;
  scan_profile(0);

  disable_event();
  load_settings_to_dialog();
  enable_event();

  save_profile();
  String name = DllPath + "delete_me.cpp";

  demostring->SaveToFile(name);
  indent_tmpfile();
  Memo1->Lines->LoadFromFile(name);

  DeleteFile(name);
  DeleteFile(name + "~");
}

//---------------------------------------------------------------------------

void __fastcall TOptionDlg::BitBtn4Click(TObject * Sender)
{
  save_profile();
  oldprofile->LoadFromFile(DllPath + ".indent.pro");
}

//---------------------------------------------------------------------------

